==> libunwind-0.2-pre2.tar.gz: libunwind-0.2-pre2/src/ia64/rse.h <==
/*
 * Copyright (C) 1998, 1999, 2002 Hewlett-Packard Co
 *	David Mosberger-Tang <davidm@hpl.hp.com>
 *
 * Register stack engine related helper functions.  This file may be
 * used in applications, so be careful about the name-space and give
 * some consideration to non-GNU C compilers (though __inline__ is
 * fine).
 */
#ifndef RSE_H
#define RSE_H

#include <stdint.h>

static __inline__ uint64_t
ia64_rse_slot_num (uint64_t addr)
{
	return (addr >> 3) & 0x3f;
}

/*
 * Return TRUE if ADDR is the address of an RNAT slot.
 */
static __inline__ uint64_t
ia64_rse_is_rnat_slot (uint64_t addr)
{
	return ia64_rse_slot_num (addr) == 0x3f;
}

/*
 * Returns the address of the RNAT slot that covers the slot at
 * address SLOT_ADDR.
 */
static __inline__ uint64_t
ia64_rse_rnat_addr (uint64_t slot_addr)
{
	return slot_addr | (0x3f << 3);
}

/*
 * Calcuate the number of registers in the dirty partition starting at
 * BSPSTORE with a size of DIRTY bytes.  This isn't simply DIRTY
 * divided by eight because the 64th slot is used to store ar.rnat.
 */
static __inline__ uint64_t
ia64_rse_num_regs (uint64_t bspstore, uint64_t bsp)
{
	uint64_t slots = (bsp - bspstore) >> 3;

	return slots - (ia64_rse_slot_num(bspstore) + slots)/0x40;
}

/*
 * The inverse of the above: given bspstore and the number of
 * registers, calculate ar.bsp.
 */
static __inline__ uint64_t
ia64_rse_skip_regs (uint64_t addr, long num_regs)
{
	long delta = ia64_rse_slot_num(addr) + num_regs;

	if (num_regs < 0)
		delta -= 0x3e;
	return addr + ((num_regs + delta/0x3f) << 3);
}

#endif /* RSE_H */

==> libunwind-0.92.tar.gz: libunwind-0.92/src/ia64/rse.h <==
/*
 * Copyright (C) 1998, 1999, 2002, 2003 Hewlett-Packard Co
 *	David Mosberger-Tang <davidm@hpl.hp.com>
 *
 * Register stack engine related helper functions.  This file may be
 * used in applications, so be careful about the name-space and give
 * some consideration to non-GNU C compilers (though __inline__ is
 * fine).
 */
#ifndef RSE_H
#define RSE_H

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <inttypes.h>

static inline uint64_t
ia64_rse_slot_num (uint64_t addr)
{
	return (addr >> 3) & 0x3f;
}

/*
 * Return TRUE if ADDR is the address of an RNAT slot.
 */
static inline uint64_t
ia64_rse_is_rnat_slot (uint64_t addr)
{
	return ia64_rse_slot_num (addr) == 0x3f;
}

/*
 * Returns the address of the RNAT slot that covers the slot at
 * address SLOT_ADDR.
 */
static inline uint64_t
ia64_rse_rnat_addr (uint64_t slot_addr)
{
	return slot_addr | (0x3f << 3);
}

/*
 * Calcuate the number of registers in the dirty partition starting at
 * BSPSTORE and ending at BSP.  This isn't simply (BSP-BSPSTORE)/8
 * because every 64th slot stores ar.rnat.
 */
static inline uint64_t
ia64_rse_num_regs (uint64_t bspstore, uint64_t bsp)
{
	uint64_t slots = (bsp - bspstore) >> 3;

	return slots - (ia64_rse_slot_num(bspstore) + slots)/0x40;
}

/*
 * The inverse of the above: given bspstore and the number of
 * registers, calculate ar.bsp.
 */
static inline uint64_t
ia64_rse_skip_regs (uint64_t addr, long num_regs)
{
	long delta = ia64_rse_slot_num(addr) + num_regs;

	if (num_regs < 0)
		delta -= 0x3e;
	return addr + ((num_regs + delta/0x3f) << 3);
}

#endif /* RSE_H */
